import MicroRNAClassification
import GistRunner
import Vienna
from PyML.classifiers.svm import loadSVM
import SnorreCutSetPredictor
from datetime import datetime
import math
import sys
from PyML import *
import MicroRNADetails
import getopt
import argparse
from pylab import *
import numpy.numarray as na 
import FeatureManagement
import MicroRNAStats
import PyMLManager
import MicroRNASVM

#Different variables
CLASSIFICATOR_DIR = "/home/frederho/repo02/cgi-bin/precursor/data/"
CLASSIFICATOR_NAME = "FeatureVersion5.0_Sanger8.0_Algorithms7.0"








class EvaluationError:
	def __init__(self):
		pass


#Method for predicting a one sequences. Input is the sequence and an optional id. Output is the list [cut, score]
#Example-input:	'uagcuccuuaccgugcucucauugccgccuccccaccucccgcuccagcccugccggggcuaaagugcugacagugcagauagugguccucuccgugcuaccgcacuguggguacuugcugcuccagcagggcacgcacagcguccguggagggaaaggccuuuuccccacuucuuaacc', 1
#Example-output:	[[60, 121], 0.89645200000000003]
def ProcessingSiteSVM(seq, ID = None, returnonlybestscore=1):
	if ID == None:
		ID = "Sequence_"+str(datetime.now()).replace(' ','_').replace(':','-')
	res = ProcessingSiteSVMBatch({ID:seq}, returnonlybestscore)
	return res

#Method for predicting a batch of sequences. Input is a map of Id:Sequence. Output is a map of Id:[cut, score]
#Example-input:	{'hsa-mir-106b':'uagcuccuuaccgugcucucauugccgccuccccaccucccgcuccagcccugccggggcuaaagugcugacagugcagauagugguccucuccgugcuaccgcacuguggguacuugcugcuccagcagggcacgcacagcguccguggagggaaaggccuuuuccccacuucuuaacc', 'hsa-mir-412':'ggaggcuggggcaccucggggaaggacgccggcaucagcaccauucugggguacggggauggauggucgaccaguuggaaaguaauuguuucuaauguacuucaccugguccacuagccguccguauccgcugcagccuguggggccugcgggccggggagccgaucgcgcuucagcuca'}, 1
#Example-output:	{'hsa-mir-106b': [[60, 121], 0.89645200000000003], 'hsa-mir-412': [[60, 125], 0.77706600000000003]}

def ProcessingSiteSVMBatch(seqs, returnonlybestscore=1):

	result = PyMLManager.PyMLManager.PyMLRunner(seqs, 'hsa', '400')
	
def GetVerifiedPrecursors():
	precs = SnorreCutSetPredictor.GetPrecursorsFixedLength('mmu', 'all')

	verifiedprecs = []

	content = [line.lower().strip('\n').replace('-5p','') for line in open('Verified-mmu.data','r')]

	for verified in content:
		if '-3p' in verified:
			content.remove(verified)
	
	for prec in precs:
		if prec.id in content:
			verifiedprecs.append(prec)

	return verifiedprecs

def GetFalsePrecursors():
	precs = SnorreCutSetPredictor.GetPrecursorsFixedLength('mmu', 'all')

	failedPrecs = []

	content = [line.lower().strip('\n').replace('-5p','') for line in open('failed-mmu.data','r')]

	for failed in content:
		if '-3p' in failed:
			content.remove(failed)

	for prec in precs:
		if prec.id in content:
			failedPrecs.append(prec)

	return failedPrecs
	

def RunFeatureTests(species, number, featureStart, featureLength):
	f = FeatureManagement.FeatureManagement()
	s = MicroRNAStats.MicroRNAStatistics()
	p = PyMLManager.PyMLManager()
	resultlist = []
	resultsdict = {}
	rocscores = []
	
	precs = SnorreCutSetPredictor.GetPrecursorsFixedLength(species, number)

	verifiedprecs = GetVerifiedPrecursors()

	tenFold = GistRunner.TenFold()
	folds = GistRunner.TenFold.Get10FoldWithFamilyInSameFold(tenFold,verifiedprecs)
	#folds = GistRunner.TenFold.Get10FoldWithFamilyInSameFold(tenFold,precs)
	for num in range(len(folds)):
		trainingset = None
		for i in range(len(folds)):
			if i != num:
				if trainingset == None:
					trainingset = folds[i]
				else:
					trainingset = trainingset + folds[i]
		[posVec, negVec] = p.GetFeaturesAndLables(trainingset)
	
		values = None
		[posVec, negVec, values] = f.FeatureScaling(posVec, negVec, values)

		newFeature = f.NewFeature(posVec)
		newFeature2 = f.NewFeature(negVec)
		
			
		#pos = MakeDict(posVec,'+1')
		pos = p.MakeDict(newFeature,'+1')
		
		#Removing specific features
		
		newPos = f.RemoveFeature(pos,featureStart,featureLength)
		#Creating a dictionary for all negative vectors
		#neg = MakeDict(negVec,'-1')
		neg = p.MakeDict(newFeature2, '-1')
		
		newNeg = f.RemoveFeature(neg,featureStart,featureLength)
		#Transforming the dictionary to PyML format and writing it to a file

		p.WriteDictToFile(newPos,newNeg)
		#WriteDictToFile(pos,neg)
		#Reading the file containing the positive and negative vectors and putting it into a PyML container
		data = SparseDataSet(CLASSIFICATOR_DIR+'PyMLtrainingData.data')
		#Normalizing the data set
		data.normalize(2)

		[pos, neg] = p.GetFeaturesAndLables(folds[num])
		
		[pos, neg, values] = f.FeatureScaling(pos, neg, values)
		
		testData = pos + neg

		newTestData = f.NewFeature(testData)
		newTestData = p.CreateVectorDataSet(newTestData)
		#testData = CreateVectorDataSet(testData)
		#newTestData= RemoveFeature(testData,featureStart,featureLength)
		newTestData = f.RemoveFeature(newTestData, featureStart, featureLength)
		#testData = CreateVectorDataSet(testData)
		p.WriteVectorDataSet(newTestData)
		#WriteVectorDataSet(testData)
		#Putting the test data set into a sparse data set container used by PyML
		testData =SparseDataSet(CLASSIFICATOR_DIR+'testData.data')
		#normalizing the data set
		testData.normalize(2)
		
		#Running PyMLtrain on the dataset
		trainedSVM = p.PyMLTrain(data)
		#Classifying the dataset
		results = p.PyMLClassify(trainedSVM, data, testData)
		resultsdict[num] = results
		rocscores.append(results.getROC())
		decisionfunction = results.getDecisionFunction()
		patternID = results.getPatternID
		idMap = IdMap(pos, neg, decisionfunction)
		
		
					
		resultlist.append(idMap)
	cutsites = s.GetCutSites(resultlist)
	statistics = s.Statistics(resultlist,cutsites)
	f = open(CLASSIFICATOR_DIR+'idMap.data','w')
	for i in range(len(resultlist)):
		f.write('fold ' + str(i))
		f.write(str(resultlist[i]))
		f.write('\n\n')
	f.close()

	f = open(CLASSIFICATOR_DIR+ 'results.data','w')
	for key in statistics.keys():
		f.write(key+ ";" )
		f.write(str(statistics[key]))
		f.write('\n\n')
	f.close()
	print rocscores
	return resultsdict	
	

def FeatureElimination(species, number):
	f = FeatureManagament.FeatureManangement()
	p = PyMLManager.PyMLManager()
	initPrecs = SnorreCutSetPredictor.GetPrecursorsFixedLength(species, number)

	verifiedprecs = GetVerifiedPrecursors()
	[posVec,negVec] = p.GetFeaturesAndLables(initPrecs)

	values = None
	[posVec, negVec, values] = f.FeatureScaling(posVec, negVec, values)

	pos = p.MakeDict(posVec,'+1')
	#Creating a dictionary for all negative vectors
	neg = p.MakeDict(negVec,'-1')
	#Transforming the dictionary to PyML format and writing it to a file
	p.WriteDictToFile(pos,neg)
	#Reading the file containing the positive and negative vectors and putting it into a PyML container
	data = SparseDataSet(CLASSIFICATOR_DIR+'PyMLtrainingData.data')
	#Normalizing the data set
	data.normalize(2)
	
	rfe = featsel.RFE()
	rfe.select(data)
	print data.featureID

def TestVerifiedOnFailed(species, number):
	f = FeatureManagement.FeatureManagement()
	p = PyMLManager.PyMLManager()
	s = MicroRNAStats.MicroRNAStatistics()
	resultlist = []
	resultsdict = {}

	initPrecs = SnorreCutSetPredictor.GetPrecursorsFixedLength(species, number)
	failedPrecs = GetFalsePrecursors() 
	correctlyPredictedPrecs = MicroRNASVM.GetCorrectlyPredictedPrecursors()
	
	correctIDList = [prec.id for prec in correctlyPredictedPrecs]
	
	verifiedPrecs = GetVerifiedPrecursors()
	trainingSet = []
	for prec in verifiedPrecs:
		if not prec.id in correctIDList:
			trainingSet.append(prec)


	
	
	[posVec, negVec] = p.GetFeaturesAndLables(failedPrecs)

	values = None
	[posVec, negVec, values] = f.FeatureScaling(posVec, negVec, values)
	
	pos = p.MakeDict(posVec,'+1')
	#Creating a dictionary for all negative vectors
	neg = p.MakeDict(negVec,'-1')
	#Transforming the dictionary to PyML format and writing it to a file
	p.WriteDictToFile(pos,neg)
	#Reading the file containing the positive and negative vectors and putting it into a PyML container
	data = SparseDataSet(CLASSIFICATOR_DIR+'PyMLtrainingData.data')
	#Normalizing the data set
	data.normalize(2)
	
	"""
	#Generating verified mmu precs

	verifiedmmu = []
	Verifiedcontent = [line.lower().strip('\n').strip('-5p').strip('-3p').strip('mmu-mir').strip('\n') for line in open('Verified-mmu.data','r')]
	mmuprecs = SnorreCutSetPredictor.GetPrecursorsFixedLength('mmu', 'all')
	for prec in mmuprecs:
		precid = prec.id
		for mmu in veridiedcontent:
			if precid == mmu:
				verifiedmmu.append(prec)
	#generating hsa minus verified mmu
	Verifiedcontent = [line.lower().strip('\n').strip('-5p').strip('-3p').strip('mmu-mir').strip('\n') for line in open('Verified-mmu.data','r')]
	hsaPrecs = SnorreCutSetPredictor.GetPrecursorsFixedLength('hsa', 'all')
	hsaPrecsMinusMMU = []
	for prec in hsaPrecs:
		precid =  prec.id.strip('hsa-mir')
		inTable = False
		for mmu in Verifiedcontent:
			if precid == mmu:
				inTable = True
		if inTable == False:
			hsaPrecsMinusMMU.append(prec)
	"""


	"""
	content = [line.lower().strip('\n') for line in open('failed-mmu.data','r')]
	for failed in content:
		if '-3p' in failed:
			content.remove(failed)
		if '-5p' in failed:
			content.strip('-5p')

	for Id in content:
		for prec in initPrecs :
			if prec.id == Id:
				print prec.id
				failedPrecs.append(prec)
				break
	"""
	
	[pos, neg] = p.GetFeaturesAndLables(correctlyPredictedPrecs)
	
	[pos, neg, values] = f.FeatureScaling(pos, neg, values)
	
	testData = pos + neg 
	testData = p.CreateVectorDataSet(testData)
	p.WriteVectorDataSet(testData)
	#Putting the test data set into a sparse data set container used by PyML
	testData =SparseDataSet(CLASSIFICATOR_DIR+'testData.data')
	#normalizing the data set
	testData.normalize(2)

	trainedSVM = p.PyMLTrain(data)
	#Classifying the dataset
	results = p.PyMLClassify(trainedSVM, data, testData)
	resultsdict[0] = results
	decisionfunction = results.getDecisionFunction()
	patternID = results.getPatternID
	idMap = IdMap(pos, neg, decisionfunction)

	resultlist.append(idMap)
	cutsites = s.GetCutSites(resultlist)
	print "Getting statistics"
	statistics = s.Statistics(resultlist,cutsites)

def Run10FoldCrossValidation(species, number):
	f = FeatureManagement.FeatureManagement()
	s = MicroRNAStats.MicroRNAStatistics()
	p = PyMLManager.PyMLManager()

	rocscores = []
	resultlist = []
	resultsdict = {}
	
	#precs = SnorreCutSetPredictor.GetPrecursorsFixedLength(species, number)
	verifiedprecs = GetVerifiedPrecursors()

	tenFold = GistRunner.TenFold()
	folds = GistRunner.TenFold.Get10FoldWithFamilyInSameFold(tenFold,verifiedprecs)
	#folds = GistRunner.TenFold.Get10FoldWithFamilyInSameFold(tenFold,precs)

	for num in range(len(folds)):
		trainingset = None
		for i in range(len(folds)):
			if i != num:
				if trainingset == None:
					trainingset = folds[i]
				else:
					trainingset = trainingset + folds[i]

		[posVec, negVec] = p.GetFeaturesAndLables(trainingset)		



		values = None
		[posVec, negVec, values] = f.FeatureScaling(posVec, negVec, values)
		
		
		pos = p.MakeDict(posVec,'+1')
		#Creating a dictionary for all negative vectors
		neg = p.MakeDict(negVec,'-1')
		#Transforming the dictionary to PyML format and writing it to a file
		p.WriteDictToFile(pos,neg)
		#Reading the file containing the positive and negative vectors and putting it into a PyML container
		data = SparseDataSet(CLASSIFICATOR_DIR+'PyMLtrainingData.data')
		#Normalizing the data set
		data.normalize(2)

		[pos, neg] = p.GetFeaturesAndLables(folds[num])

		[pos, neg, values] =f. FeatureScaling(pos, neg, values)
				
		testData = pos + neg 
		testData = p.CreateVectorDataSet(testData)
		p.WriteVectorDataSet(testData)
		#Putting the test data set into a sparse data set container used by PyML
		testData =SparseDataSet(CLASSIFICATOR_DIR+'testData.data')
		#normalizing the data set
		testData.normalize(2)
		
		#Running PyMLtrain on the dataset
		trainedSVM = p.PyMLTrain(data)
		#Classifying the dataset
		results = p.PyMLClassify(trainedSVM, data, testData)
		resultsdict[num] = results
		rocscores.append(results.getROC())
		decisionfunction = results.getDecisionFunction()
		patternID = results.getPatternID
		idMap = IdMap(pos, neg, decisionfunction)
		resultlist.append(idMap)

		#This section is used to generate data for each spesific fold. Just change fold = x to get fold x
		"""
		#Fjern dette naar grafene er laget
		fold = 9
		if num == fold:
			resultlist2 =[]
			resultlist2.append(idMap)
			cutSiteList = []
			i = 0
			temp = []
			cutsites = {}
			for j in range(len(idMap)): temp.append(idMap[j][0])
			while idMap[i][2] == 1:
				indices = [k for k, x in enumerate(temp) if temp[k] == idMap[i][0]]
				cutsites[idMap[i][0]] = indices
				i = i + 1
				cutSiteList.append(cutsites)

			statistic = Statistics(resultlist2, cutSiteList)
		#fjer alt ned til denne kommentaren
		"""
		print "==========================================="
		print num
		print "==========================================="
	cutsites = s.GetCutSites(resultlist)
	statistics = s.Statistics(resultlist,cutsites)

	
	f = open(CLASSIFICATOR_DIR+'idMap.data','w')
	for i in range(len(resultlist)):
		f.write('fold ' + str(i))
		f.write(str(resultlist[i]))
		f.write('\n\n')
	f.close()

	f = open(CLASSIFICATOR_DIR+ 'results.data','w')
	for key in statistics.keys():
		f.write(key+ ";" )
		f.write(str(statistics[key]))
		f.write('\n\n')
	f.close()
	print rocscores
	return resultsdict

    #Generates an information container describing if a vector is positive or negative and the corresponding decsision function.
def IdMap(pos, neg, decisionFunction):
	idMap = []
	for i in range(len(pos)):
		[Id, position] = pos[i][0].split('#')
		idMap.append([Id, position, 1, None, pos[i]])

	for i in range(len(neg)):
		[Id, position] = neg[i][0].split('#')
		idMap.append([Id, position, 0,  None, neg[i]])

	for i in range(len(decisionFunction)):
		idMap[i][3] = decisionFunction[i]

	return idMap


def main():

	
	#Her skal det implementeres getopt options, enn saa lenge 10Fold og streng input
	param = sys.argv
	if len(param)>1 and param[0]=="MicroRNAProcessingSiteSVM.py":
		if param[1] == "10Fold":
			print "I am running 10 Fold with "+param[2] + " " + param[3]
			Run10FoldCrossValidation(param[2], param[3])
		elif param[1] == "FeatureTest":
			print "Running feature test " +param[2]+" " + param[3]
			RunFeatureTests('mmu','all', int(param[2]), int(param[3]))
		elif param[1] == "Test":
			print "testing on set"
			TestVerifiedOnFailed('mmu','all')
		else:	
			result = ProcessingSiteSVM(param[1])
			print "Score: "+str(result[1])+", predicted processing site: "+str(result[0])


if __name__ == '__main__':
	main()


#test = 'uagcuccuuaccgugcucucauugccgccuccccaccucccgcuccagcccugccggggcuaaagugcugacagugcagauagugguccucuccgugcuaccgcacuguggguacuugcugcuccagcagggcacgcacagcguccguggagggaaaggccuuuuccccacuucuuaacc'
#print ProcessingSiteSVM(test, None, 1)

#test = {'hsa-mir-106b':'uagcuccuuaccgugcucucauugccgccuccccaccucccgcuccagcccugccggggcuaaagugcugacagugcagauagugguccucuccgugcuaccgcacuguggguacuugcugcuccagcagggcacgcacagcguccguggagggaaaggccuuuuccccacuucuuaacc', 'hsa-mir-412':'ggaggcuggggcaccucggggaaggacgccggcaucagcaccauucugggguacggggauggauggucgaccaguuggaaaguaauuguuucuaauguacuucaccugguccacuagccguccguauccgcugcagccuguggggccugcgggccggggagccgaucgcgcuucagcuca'}
#print ProcessingSiteSVMBatch(test, 1)

