
import MicroRNAClassification
import GistRunner
import Vienna
from PyML.classifiers.svm import loadSVM
import SnorreCutSetPredictor
from datetime import datetime
import math
import sys
from PyML import *
import MicroRNADetails
import getopt
import argparse
from pylab import *
import numpy.numarray as na 
import FeatureManagement
import MicroRNAStats
import PyMLManager


CLASSIFICATOR_DIR = "/home/frederho/repo02/cgi-bin/precursor/data/"
CLASSIFICATOR_NAME = "FeatureVersion5.0_Sanger8.0_Algorithms7.0"
class PyMLManager:
    def PyMLRunner(self, seqs, species, size):
        s = MicroRNAStats.MicroRNAStatistics()
	precs = SnorreCutSetPredictor.GetPrecursorsFixedLength(species, size)
	#getting features and lables from the entire mirbase
	[posVec, negVec] = PyMLManager().GetFeaturesAndLables(precs)
	#Creating a dictionary for all positive vectors
	pos = PyMLManager().MakeDict(posVec,'+1')
	#Creating a dictionary for all negative vectors
	neg = PyMLManager().MakeDict(negVec,'-1')
	#Transforming the dictionary to PyML format and writing it to a file
	PyMLManager().WriteDictToFile(pos,neg)
	#Reading the file containing the positive and negative vectors and putting it into a PyML container 
	data = SparseDataSet(CLASSIFICATOR_DIR + 'PyMLtestdata.data')

	#Running PyMLtrain on the dataset
	trainedSVM = PyMLManager().PyMLTrain(data)
	#Classifying the dataset
	results = {}
	bestresult = {}
	for key in seqs.keys():
		tempdict = {}
		tempdict[key] = seqs[key]
		#Getting vectors for the sequence to be classified
		vectors = PyMLManager().CreateTestVector(tempdict)
		#Creating the TestVector Dictionary
		testData = PyMLManager().CreateVectorDataSet(vectors)	
		#Writing the dataset to file
		PyMLManager().WriteVectorDataSet(testData)
		#Putting the test data set into a sparse data set container used by PyML
		testData =SparseDataSet(CLASSIFICATOR_DIR + 'testData.data')
		results[key]  = PyMLManager().PyMLClassify(trainedSVM, data, testData)
		predicted = s.GetPredictions(results[key],vectors)
		print results[key].getGivenLabels()
		print results[key].getDecisionFunction()
		best = PyMLManager.GetBestCut(predicted)
		bestresult[key] = best

	return [results, bestresult]

    def CreateTestVector(self, seq):
	key = None
	for key in seq.keys():
		key = key
	if len(seq[key])<180:
		print "<p>Warning: Sequence "+key+" is shorter than 180 nucleotides.<br> Not all possible processing sites will be evaluated!</p>"
	candidate = PyMLManager().CreateCandidate(seq[key], key)
	wannabes = [MicroRNAClassification.MicroRNAWannabe(candidate, cut) for cut in candidate.GetPossibleCutSet()]
	if len(wannabes)==0:
		print "<p>Error: No suitable processing sites found in sequence " + key + "</p>"
		
	vectors = MicroRNAClassification.MicroRNAVectors().GetCutSetVectors(wannabes)
		
	return vectors

    def CreateVectorDataSet(self, vectors):
	vectordict = {}

	for j in range(len(vectors)):
		featuredict = {}
		temp = vectors[j]
		for i in range(len(temp)):
			#for every new vector, set the first element to the correct label
			
			featuredict[i]= temp[i]
			
			
			
		#put each vector into a dictionary containing the vectors
	
		vectordict[j]=featuredict
	return vectordict

    def WriteVectorDataSet(self, testData):
	f = open(CLASSIFICATOR_DIR + 'testData.data','w')

	for i in range(len(testData)):
		f.write(str(testData[i]).replace("0: '","'").replace("'", "").replace(',','').replace(': ',':').replace(':{',' ').replace('{','').replace('}','\n'))

	f.close()

    #Creating and training a SVM on a known dataset
    def PyMLTrain(self, data):
	filename ='SVMdata.data' 
	#Creating a SVM classifier 
	#svm = SVM(Cmode = 'classProb', optimizer = 'liblinear')
	svm = SVM(Cmode = 'classProb')
	#Training the SVM on the known data set
	svm.train(data)
	#saving the SVM
#	crossvalidation = svm.cv(data)
	svm.save(filename)
	return filename

    #classify unknown dataset using a trained SVM
    def PyMLClassify(self, name, data, testData):
	#Load the trained SVM
	trainedSVM = loadSVM(name,data)
	#run classificatin on the dataset using the loaded SVM
	results = trainedSVM.test(testData)
	#returning a result object containing a different information such as decision functions etc
	return results

    #Writing dictionary to a file in a format that PyML is able to understand
    def WriteDictToFile(self, pos,neg):

	f = open(CLASSIFICATOR_DIR +'PyMLtrainingData.data','w')	

        #for all positive vectors, label it +1 and list all features in the format of Feature#:Featurevalue
	for i in range(len(pos)):
		f.write(str(pos[i]).replace("'", "").replace(', ',' ').replace(': ',':').replace(':{',' ').replace('{','').replace('}','\n').replace("0:miRNA","miRNA"))

        #for all negative vectors, label it +1 and list all features in the format of Feature#:Featurevalue
	for j in range(len(neg)):
		f.write(str(neg[j]).replace("'", "").replace(', ',' ').replace(': ',':').replace(':{',' ').replace('{','').replace('}','\n').replace("0:miRNA","miRNA"))

	f.close()

    #Put the vector into a dictionary
    def WriteTestDictToFile(self, pos,neg):
        
	f = open(CLASSIFICATOR_DIR +'PyMLtestData.data','w')	
        
        #for all positive vectors, label it +1 and list all features in the format of Feature#:Featurevalue
	for i in range(len(pos)):
            f.write(str(pos[i]).replace("'", "").replace(', ',' ').replace(': ',':').replace(':{',' ').replace('{','').replace('}','\n').replace("0:miRNA","miRNA"))
            
        #for all negative vectors, label it +1 and list all features in the format of Feature#:Featurevalue
	for j in range(len(neg)):
            f.write(str(neg[j]).replace("'", "").replace(', ',' ').replace(': ',':').replace(':{',' ').replace('{','').replace('}','\n').replace("0:miRNA","miRNA"))
            
        f.close()

    def MakeDict(self, vectors,label):
	
	vectordict = {}
	
	for j in range(len(vectors)):
		featuredict = {}
		temp = vectors[j]
		for i in range(len(temp)):
			#for every new vector, set the first element to the correct label
			if i ==  0:
                            featuredict[i]= temp[i].replace(',','') + ',, ' +  label
			#for all other elements list features on the format feature#:feature value
			else:
				featuredict[i]= temp[i]
		#put each vector into a dictionary containing the vectors
		vectordict[j] = featuredict
	return vectordict

    #Getting features and lables to be used for the known set
    def GetFeaturesAndLables(self, precs):
	
	
	precs = SnorreCutSetPredictor.FilterPrecursors(precs, "8.1_Only", "None")
	#Getting precursors of length 180
	precs  = SnorreCutSetPredictor._FixPrecursorLengths(precs, 180)
	#Getting precursors of length 110 (this is needed due to the unstable folding prediction for 180 
 	precs2 = SnorreCutSetPredictor._FixPrecursorLengths(precs, 110)
	#Getting Precursor candidates
	sets   = SnorreCutSetPredictor.ConvertPrecursorsToTrainingSets(precs, precs2)
	#Generating positive vector set
	posVec = MicroRNAClassification.MicroRNAVectors().GetCutSetVectors(sets['pos'])
	#Generating negative vector set
	negVec = MicroRNAClassification.MicroRNAVectors().GetCutSetVectors(sets['neg'])
	
	return (posVec,negVec)
    
    def CreateCandidate(self, seq, id):
	fold = Vienna.RunRNAfold(seq)
	diff = float(len(seq)-110)/2
	fold110 = Vienna.RunRNAfold(seq[int(math.ceil(diff)):-int(math.floor(diff))])
	return MicroRNAClassification.MicroRNACandidate(seq, fold, id, '', '', '', fold110)
